@@ -1,6 +1,6 @@
 import importlib
 
-from importlib.metadata import entry_points
+from importlib_metadata import entry_points
 
 registered_plugins = []
 
