@@ -1,4 +1,5 @@
 import png
+import sys
 import time
 import cmap
 import uuid
@@ -8,22 +9,26 @@ import warnings
 
 import numpy as np
 import pygfx as gfx
+import trimesh as tm
 
-from functools import wraps, lru_cache, partial
+from pathlib import Path
 from collections import OrderedDict
+from functools import wraps, lru_cache, partial
 
-from wgpu.gui.auto import WgpuCanvas, run
-from wgpu.gui.offscreen import WgpuCanvas as WgpuCanvasOffscreen
+from rendercanvas.offscreen import OffscreenRenderCanvas
 
 from .visuals import mesh2gfx, volume2gfx, points2gfx, lines2gfx, text2gfx
 from .conversion import get_converter
 from . import utils, config
 
 
-__all__ = ["Viewer"]
+__all__ = ["Viewer", "viewers"]
 
 logger = config.get_logger(__name__)
 
+# This keeps track of open viewers
+viewers = []
+
 AUTOSTART_EVENT_LOOP = True
 
 # TODO
@@ -44,53 +49,64 @@ def update_viewer(legend=True, bounds=True):
 
         @wraps(func)
         def inner(*args, **kwargs):
+            # Run function first
             func(*args, **kwargs)
-
-            # Always clear the cached objects dictionary
-            args[0]._objects.cache_clear()
-
-            if legend:
-                if getattr(args[0], "controls", None):
-                    args[0].controls.update_legend()
-                if getattr(args[0], "widget", None):
-                    if args[0].widget.toolbar:
-                        args[0].widget.toolbar.update_legend()
-            if bounds:
-                if getattr(args[0], "show_bounds", False):
-                    args[0].update_bounds()
+            update_helper(viewer=args[0], legend=legend, bounds=bounds)
 
         return inner
 
     return outer
 
 
+def update_helper(viewer, legend=True, bounds=True):
+    """Helper function to update legend and other properties."""
+    # Always clear the cached objects dictionary
+    viewer._objects.cache_clear()
+
+    if legend:
+        if getattr(viewer, "controls", None):
+            viewer.controls.update_legend()
+        if getattr(viewer, "widget", None):
+            if viewer.widget.toolbar:
+                viewer.widget.toolbar.update_legend()
+    if bounds:
+        if getattr(viewer, "show_bounds", False):
+            viewer.update_bounds()
+
+    # Any time we update the viewer, we should set it to stale
+    viewer._render_stale = True
+    viewer.canvas.request_draw()
+
+
 class Viewer:
     """PyGFX 3D viewer.
 
     Parameters
     ----------
-    offscreen : bool, optional
+    offscreen : bool
                 If True, will use an offscreen Canvas. Useful if you only
                 want a screenshot.
-    title :     str, optional
+    title :     str
                 Title of the viewer window.
-    max_fps :   int, optional
+    max_fps :   int
                 Maximum frames per second to render.
     size :      tuple, optional
                 Size of the viewer window.
-    camera :    "ortho" | "perspective", optional
+    camera :    "ortho" | "perspective"
                 Type of camera to use. Defaults to "ortho". Note you can always
                 change the camera type by adjust the `Viewer.camera.fov` attribute
                 (0 = ortho, >0 = perspective).
     control :   "trackball" | "panzoom" | "fly" | "orbit"
                 Controller type to use. Defaults to "trackball".
-    show :      bool
-                Whether to immediately show the viewer. A few notes:
-                 1. This has no effect in Jupyter. There you will have to call ``.show()``
-                    manually on the last line of a cell for the viewer to appear.
-                 2. When running in a non-interactive script or REPL, you have to also start
-                    the event loop manually.
-                See the `Viewer.show()` method for more information.
+    show :      "auto" (default) | bool
+                Whether to immediately show the viewer. When set to "auto" (default),
+                will immmediately show the viewer if:
+                 - we are in a Jupyter environment
+                 - we are in an iPython session and we can hook into an iPython event loop
+                If neither of the above applies or `show=False`, you will have to manually run
+                `Viewer.show()`. This gives you the chance to add objects to the viewer
+                before it is shown and the blocking event loop is started.
+                The `show` parameter is ignored if `offscreen` is True.
     **kwargs
                 Keyword arguments are passed through to ``WgpuCanvas``.
 
@@ -98,6 +114,7 @@ class Viewer:
 
     # Palette used for assigning colors to objects
     palette = "seaborn:tab10"
+    highlight_color = "yellow"
 
     def __init__(
         self,
@@ -110,8 +127,13 @@ class Viewer:
         show=True,
         **kwargs,
     ):
+        # We need to import WgpuCanvas before we (potentially) start the event loop
+        # If we don't we get a segfault.
+        if not offscreen:
+            from rendercanvas.auto import RenderCanvas
+
         # Check if we're running in an IPython environment
-        if utils._type_of_script() == "ipython":
+        if utils._type_of_script() == "ipython" and not offscreen:
             ip = get_ipython()  # noqa: F821
             if not ip.active_eventloop:
                 if AUTOSTART_EVENT_LOOP:
@@ -122,7 +144,7 @@ class Viewer:
                             "started a GUI event loop. We've started one for you using the "
                             "Qt6 backend."
                         )
-                    except ModuleNotFoundError:
+                    except (ModuleNotFoundError, ImportError):
                         raise ValueError(
                             "Looks like you're running in an IPython environment but haven't "
                             "started a GUI event loop. We tried to start one for you using the "
@@ -139,7 +161,9 @@ class Viewer:
         self._title = title
 
         # Update some defaults as necessary
-        defaults = {"title": title, "max_fps": max_fps, "size": size}
+        defaults = {"title": title, "max_fps": max_fps}
+        if size is not None:
+            defaults["size"] = size
         defaults.update(kwargs)
 
         # If we're running in headless mode (primarily for tests on CI) we will
@@ -149,9 +173,9 @@ class Viewer:
             return
 
         if not offscreen:
-            self.canvas = WgpuCanvas(**defaults)
+            self.canvas = RenderCanvas(**defaults)
         else:
-            self.canvas = WgpuCanvasOffscreen(**defaults)
+            self.canvas = OffscreenRenderCanvas(**defaults)
 
         # There is a bug in pygfx 0.1.18 that causes the renderer to crash
         # when using a Jupyter canvas without explicitly setting the pixel_ratio.
@@ -171,21 +195,21 @@ class Viewer:
 
         # A strong point light form front/top/left
         self.scene.add(gfx.PointLight(intensity=4))
-        self.scene.children[-1].shadow.bias = 0.0000005 # this helps with shadow acne
+        self.scene.children[-1].shadow.bias = 0.0000005  # this helps with shadow acne
         self.scene.children[-1].local.x = -1000000  # move to the left
         self.scene.children[-1].local.y = -1000000  # move up
         self.scene.children[-1].local.z = -1000000  # move light forward
 
         # A weaker point light from the back
         self.scene.add(gfx.PointLight(intensity=1))
-        self.scene.children[-1].shadow.bias = 0.0000005 # this helps with shadow acne
+        self.scene.children[-1].shadow.bias = 0.0000005  # this helps with shadow acne
         self.scene.children[-1].local.x = 1000000  # move to the left
         self.scene.children[-1].local.y = 1000000  # move up
         self.scene.children[-1].local.z = 1000000  # move light forward
 
         # Set up a default background
-        self._background = gfx.BackgroundMaterial((0, 0, 0))
-        self.scene.add(gfx.Background(None, self._background))
+        self._background = gfx.Background(None, gfx.BackgroundMaterial((0, 0, 0)))
+        self.scene.add(self._background)
 
         # Add camera
         if camera == "ortho":
@@ -211,15 +235,25 @@ class Viewer:
         self.overlay_camera = gfx.NDCCamera()
         self.overlay_scene = gfx.Scene()
 
+        # Setup transform gizmo
+        self.transform_gizmo = None
+
         # Stats
         self.stats = gfx.Stats(self.renderer)
         self._show_fps = False
 
         # Setup key events
         self._key_events = {}
-        self._key_events["1"] = lambda: self.set_view("XY")
-        self._key_events["2"] = lambda: self.set_view("XZ")
-        self._key_events["3"] = lambda: self.set_view("YZ")
+        self._key_events["1"] = lambda: self.set_view("XY")  # frontal view
+        self._key_events["2"] = lambda: self.set_view("XZ")  # lateral view
+        self._key_events["3"] = lambda: self.set_view("YZ")  # top view
+        self._key_events[("1", ("Shift",))] = lambda: self.set_view("-XY")  # back view
+        self._key_events[("2", ("Shift",))] = lambda: self.set_view(
+            "-XZ"
+        )  # other lateral view
+        self._key_events[("3", ("Shift",))] = lambda: self.set_view(
+            "-YZ"
+        )  # bottom view
         self._key_events["f"] = lambda: self._toggle_fps()
         self._key_events["c"] = lambda: self._toggle_controls()
 
@@ -241,10 +275,14 @@ class Viewer:
         self._shadows = False
         self._animations = {}
         self._animations_flagged_for_removal = []
+        self._animations_frame_counter = 0
         self._on_double_click = None
         self._on_hover = None
         self._objects_pickable = False
         self._selected = []
+        self._render_trigger = "continuous"
+
+        viewers.append(self)
 
         # This starts the animation loop
         if show and not self._is_jupyter:
@@ -252,12 +290,24 @@ class Viewer:
 
     def _animate(self):
         """Run the rendering loop."""
+        rm = self.render_trigger
+
         # First run the user animations
-        # Note: we're iterating over the list because the user might add / remove
+        self._animations_frame_counter += 1
+        if self._animations_frame_counter == sys.maxsize:  # reset to avoid overflow
+            self._animations_frame_counter = 0
+        # N.B. we're iterating over the list because the user might add / remove
         # animations during the loop
-        for i, (func, on_error) in enumerate(list(self._animations.items())):
+        for i, (func, (on_error, run_every, req_render)) in enumerate(
+            list(self._animations.items())
+        ):
+            # Skip if we're not supposed to run this frame
+            if run_every and (self._animations_frame_counter % run_every) != 0:
+                continue
             try:
                 func()
+                if req_render:
+                    self._render_stale = True
             except BaseException as e:
                 if on_error == "raise":
                     raise e
@@ -273,18 +323,39 @@ class Viewer:
             self._animations.pop(f)
         self._animations_flagged_for_removal = []
 
+        # Now check if we need to render the scene
+        if rm == "active_window":
+            # Note to self: we need to explore how to do this with different backends / Window managers
+            # Not sure if this will work with e.g. Jupyter (does it know when the notebook is active?)
+            if hasattr(self.canvas, "isActiveWindow"):
+                if not self.canvas.isActiveWindow():
+                    self.canvas.request_draw()
+                    return
+        elif rm == "reactive":
+            # If the scene is not stale, we can skip rendering
+            if not getattr(self, "_render_stale", False):
+                self.canvas.request_draw()
+                return
+
         # Now render the scene
         if self._show_fps:
             with self.stats:
                 self.renderer.render(self.scene, self.camera, flush=False)
+                if self.transform_gizmo:
+                    self.renderer.render(self.transform_gizmo, self.camera, flush=False)
                 self.renderer.render(
                     self.overlay_scene, self.overlay_camera, flush=False
                 )
             self.stats.render()
         else:
             self.renderer.render(self.scene, self.camera, flush=False)
+            if self.transform_gizmo:
+                self.renderer.render(self.transform_gizmo, self.camera, flush=False)
             self.renderer.render(self.overlay_scene, self.overlay_camera)
 
+        # Set stale to False
+        self._render_stale = False
+
         self.canvas.request_draw()
 
     def _next_color(self):
@@ -322,16 +393,61 @@ class Viewer:
 
     @property
     def blend_mode(self):
-        """Render blend mode.
+        """Deprecated! Render blend mode.
 
-        This is a simple shortcut, see `Viewer.renderer.blend_mode` for more information.
+        This property has been deprecated. Please use `Viewer.set_alpha_mode()` instead.
 
         """
-        return self.renderer.blend_mode
+        raise DeprecationWarning(
+            "The 'blend_mode' property is deprecated. Please use 'Viewer.set_alpha_mode()' instead."
+        )
 
-    @blend_mode.setter
-    def blend_mode(self, mode):
-        self.renderer.blend_mode = mode
+    @property
+    def render_trigger(self):
+        """Determines when the scene is (re)rendered.
+
+        By default, we leave it to the renderer to decide when to render the scene.
+        You can adjust that behaviour by setting render mode to:
+         - "continuous" (default): leave it to the renderer to decide when to render the scene
+         - "reactive": rendering is only triggered when the scene changes
+         - "active_window": rendering is only done when the window is active; this currently
+           only works with the PySide backend
+
+        """
+        return self._render_trigger
+
+    @render_trigger.setter
+    def render_trigger(self, mode):
+        valid = ("continuous", "active_window", "reactive")
+        if mode not in valid:
+            raise ValueError(f"Unknown render mode: {mode}. Must be one of {valid}.")
+
+        # No need to do anything if the value is the same
+        if mode == getattr(self, "_render_trigger", None):
+            return
+
+        # Add/remove event handlers as necessary
+        if mode == "reactive":
+            self._set_stale_func = lambda event: setattr(self, "_render_stale", True)
+            self.renderer.add_event_handler(
+                self._set_stale_func,
+                "pointer_down",
+                "pointer_move",
+                "pointer_up",
+                "wheel",
+                # "before_render",
+            )
+        elif self._render_trigger == "reactive":
+            self.renderer.remove_event_handler(
+                self._set_stale_func,
+                "pointer_down",
+                "pointer_move",
+                "pointer_up",
+                "wheel",
+                # "before_render",
+            )
+
+        self._render_trigger = mode
 
     @property
     def controls(self):
@@ -363,42 +479,26 @@ class Viewer:
 
     @selected.setter
     def selected(self, val):
-        val = utils.make_iterable(val)
+        val = utils.make_iterable(val) if val is not None else []
 
         objects = self.objects  # grab once to speed things up
         logger.debug(f"{len(val)} objects selected ({len(self.selected)} previously)")
-        # First un-highlight neurons no more selected
+        # First un-highlight neurons which aren't selected anymore
         for s in [s for s in self._selected if s not in val]:
             for v in objects[s]:
-                if isinstance(v, gfx.Mesh):
-                    v.color = v._stored_color
-                else:
-                    v.set_data(color=v._stored_color)
+                v.material.color = v._stored_color
 
         # Highlight new additions
         for s in val:
             if s not in self._selected:
                 for v in objects[s]:
                     # Keep track of old colour
-                    v.unfreeze()
-                    v._stored_color = v.color
-                    v.freeze()
-                    if isinstance(v, gfx.Mesh):
-                        v.color = self.highlight_color
-                    else:
-                        v.set_data(color=self.highlight_color)
-
+                    v._stored_color = v.material.color
+                    v.material.color = gfx.Color(self.highlight_color)
         self._selected = list(val)
 
-        # Update legend
-        if self.show_legend:
-            self.update_legend()
-
-        # Update data text
-        # Currently only the development version of vispy supports escape
-        # character (e.g. \n)
-        t = "| ".join([f"{objects[s][0]._name} - #{s}" for s in self._selected])
-        self._data_text.text = t
+        # Update legend and set render stale (if applicable)
+        update_helper(self, legend=True, bounds=False)
 
     @property
     def size(self):
@@ -478,6 +578,36 @@ class Viewer:
         assert isinstance(v, int)
         self.canvas._subwidget._max_fps = v
 
+    @property
+    def moveable_object(self):
+        """Get/Set the object that can be moved via the transform gizmo."""
+        if self.transform_gizmo is None:
+            return None
+        return self.transform_gizmo._object_to_control
+
+    @moveable_object.setter
+    def moveable_object(self, obj):
+        if obj is None:
+            if self.transform_gizmo:
+                self.transform_gizmo._object_to_control = None
+            return
+
+        if isinstance(obj, str):
+            if obj not in self.objects:
+                raise ValueError(f"Object '{obj}' not found.")
+            elif len(self.objects[obj]) > 1:
+                raise ValueError(f"Object '{obj}' consists of multiple WorldObjects.")
+            obj = self.objects[obj][0]
+        elif not isinstance(obj, gfx.WorldObject):
+            raise TypeError(f"Expected pygfx object, got {type(obj)}")
+
+        if self.transform_gizmo is None:
+            # The transform gizmo is rendered independent of the scene (so it always stay on top)
+            self.transform_gizmo = gfx.TransformGizmo(obj)
+            self.transform_gizmo.add_default_event_handlers(self.renderer, self.camera)
+        else:
+            self.transform_gizmo._object_to_control = obj
+
     @property
     def _is_jupyter(self):
         """Check if Viewer is using Jupyter canvas."""
@@ -486,7 +616,15 @@ class Viewer:
     @property
     def _is_offscreen(self):
         """Check if Viewer is using offscreen canvas."""
-        return isinstance(self.canvas, WgpuCanvasOffscreen)
+        return isinstance(self.canvas, OffscreenRenderCanvas)
+
+    @property
+    def _window_manager(self):
+        """Which window manager is being used."""
+        try:
+            return type(self.canvas).__module__.split(".")[-1]
+        except BaseException:
+            return "na"
 
     @property
     def _object_ids(self):
@@ -561,7 +699,9 @@ class Viewer:
     def on_hover(self, v):
         valid = (None, "highlight")
         if v not in valid:
-            raise ValueError(f"Unknown value for on_hover: {v}. Must be one of {valid}.")
+            raise ValueError(
+                f"Unknown value for on_hover: {v}. Must be one of {valid}."
+            )
 
         # No need to do anything if the value is the same
         if v == self._on_hover:
@@ -588,8 +728,7 @@ class Viewer:
 
     def _highlight_on_hover_event(self, event):
         """This is the event callback for highlighting objects on hover."""
-        # print(event.time_stamp, event.pick_info)
-        # If any buttons (including mouse) are pressed (e.g. during panning) ignore the event
+        # If any buttons are pressed (e.g. mouse left during panning) ignore the event
         if event.buttons:
             return
 
@@ -603,23 +742,19 @@ class Viewer:
             return
 
         new_hover_id = [k for k, v in self.objects.items() if new_hover in v]
-        # print(f"  New Hover: {new_hover_id}")
         new_hover_id = new_hover_id[0] if new_hover_id else None
 
         # See if we need to de-highlight the current hover object
         if current_hover:
             # If the new object is the same as the current one, we don't need to do anything
             if current_hover == new_hover_id:
-                # print("  Hover hasn't changed")
                 return
             if current_hover in self.objects:
-                # print(" Unhighlighting current hover")
                 self.unhighlight_objects(current_hover)
             self._current_hover_object = None
 
         # Highlight the new object
         if new_hover_id:
-            # print("  Highlighting new hover")
             self.highlight_objects(
                 new_hover_id, color=getattr(self, "_highlight_on_hover_color", 0.2)
             )
@@ -669,7 +804,7 @@ class Viewer:
 
         self._on_double_click = v
 
-    def add_animation(self, x, on_error="remove"):
+    def add_animation(self, x, on_error="remove", run_every=None, req_render=True):
         """Add animation function to the Viewer.
 
         Parameters
@@ -681,6 +816,13 @@ class Viewer:
                     the function will be removed from the animation loop. If
                     "ignore", the error will be ignored and the function will
                     continue to be called.
+        run_every : int, optional
+                    Use to run the function every n frames.
+        req_render : bool, optional
+                    Whether this animation requires a re-render of the scene.
+                    This is mainly a flag to help the viewer to decide
+                    whether/when to trigger a render. See also the `render_trigger`
+                    property.
 
         """
         if not callable(x):
@@ -688,7 +830,7 @@ class Viewer:
 
         assert on_error in ["remove", "ignore", "raise"]
 
-        self._animations[x] = on_error
+        self._animations[x] = (on_error, run_every, req_render)
 
     def remove_animation(self, x):
         """Remove animation function from the Viewer.
@@ -744,18 +886,20 @@ class Viewer:
         # Start the animation loop
         self.canvas.request_draw(self._animate)
 
-        # If this is an offscreen canvas, we don't need to do anything
-        if isinstance(self.canvas, WgpuCanvasOffscreen):
+        # If this is an offscreen canvas, we don't need to do anything else
+        if isinstance(self.canvas, OffscreenRenderCanvas):
             return
 
         # In terminal we can just show the window
         if not self._is_jupyter:
-            # Not all backends have a show method
+            # Not all backends have a show method (e.g. GLFW does not)
             if hasattr(self.canvas, "show"):
                 self.canvas.show()
 
             if start_loop:
-                run()
+                from rendercanvas.auto import loop
+
+                loop.run()
             elif utils._type_of_script() in ("terminal", "script"):
                 logger.warning(
                     "Running in a (potentially) non-interactive terminal or script "
@@ -764,25 +908,28 @@ class Viewer:
                     "  >>> v = octarine.Viewer(show=False)\n"
                     "  >>> ...  # setup your viewer\n"
                     "  >>> v.show(start_loop=True)\n\n"
-                    "Alternatively, use WGPU's run() function:\n\n"
-                    "  >>> from wgpu.gui.auto import run\n"
+                    "Alternatively, use the loop.run() function:\n\n"
+                    "  >>> from rendercanvas.auto import loop\n"
                     "  >>> ...  # setup your viewer\n"
                     "  >>> v.show()\n"
-                    "  >>> run()\n\n"  # do not remove the \n\n here
+                    "  >>> loop.run()\n\n"  # do not remove the \n\n here
                 )
-        # For Jupyter we need to wrap the canvas in a widget
         else:
             # if not hasattr(self, 'widget'):
             from .jupyter import JupyterOutput
+            from IPython.display import display
 
             # Construct the widget
-            self.widget = JupyterOutput(
-                self,
-                use_sidecar=use_sidecar,
-                toolbar=toolbar,
-                sidecar_kwargs={"title": self._title},
-            )
-            return self.widget
+            if not hasattr(self, "widget"):
+                self.widget = JupyterOutput(
+                    self,
+                    use_sidecar=use_sidecar,
+                    toolbar=toolbar,
+                    sidecar_kwargs={"title": self._title},
+                )
+
+            # This will display the viewer right here and there
+            display(self.widget)
 
     def show_message(
         self, message, position="top-right", font_size=20, color=None, duration=None
@@ -828,9 +975,9 @@ class Viewer:
         if self._message_text not in self.overlay_scene.children:
             self.overlay_scene.add(self._message_text)
 
-        self._message_text.geometry.set_text(message)
-        self._message_text.geometry.font_size = font_size
-        self._message_text.geometry.anchor = position
+        self._message_text.set_text(message)
+        self._message_text.font_size = font_size
+        self._message_text.anchor = position
         if color is not None:
             self._message_text.material.color = cmap.Color(color).rgba
         self._message_text.material.opacity = 1
@@ -901,6 +1048,9 @@ class Viewer:
         # Remove everything but the lights and backgrounds
         self.scene.remove(*self.visuals)
 
+        # Rset the transform gizmo
+        self.transform_gizmo = None
+
     @update_viewer(legend=True, bounds=True)
     def remove_objects(self, to_remove):
         """Remove given neurons/visuals from canvas."""
@@ -924,6 +1074,18 @@ class Viewer:
         """Set to ``True`` to show bounding box."""
         return self._show_bounds
 
+    @property
+    def show_fps(self):
+        """Show frames per second."""
+        return self._show_fps
+
+    @show_fps.setter
+    def show_fps(self, v):
+        if not isinstance(v, bool):
+            raise TypeError(f"Expected bool, got {type(v)}")
+        self._show_fps = v
+        self._render_stale = True
+
     def toggle_bounds(self):
         """Toggle bounding box."""
         self.show_bounds = not self.show_bounds
@@ -1095,6 +1257,11 @@ class Viewer:
                     If True, re-center camera to all objects on canvas.
 
         """
+        if isinstance(mesh, tm.Scene):
+            for _, ob in mesh.geometry.items():
+                self.add_mesh(ob, name=name, color=color, alpha=alpha, center=False)
+            return
+
         if not utils.is_mesh_like(mesh):
             raise TypeError(f"Expected mesh-like object, got {type(mesh)}")
         if color is None:
@@ -1355,6 +1522,11 @@ class Viewer:
         if hasattr(self, "widget") and not getattr(self.widget, "_is_closed", False):
             self.widget.close(close_viewer=False)
 
+        try:
+            viewers.remove(self)
+        except ValueError:
+            pass
+
     @update_viewer(legend=True, bounds=True)
     def hide_objects(self, obj):
         """Hide given object(s).
@@ -1560,16 +1732,50 @@ class Viewer:
             for v in objects[ob]:
                 v._pinned = False
 
+    @update_viewer(legend=False, bounds=False)
+    def set_alpha_mode(self, mode, objects=None):
+        """Defines how objects' colors are blended.
+
+        With version v0.13.0 pygfx replaced the single renderer.blend_mode property with
+        customizable per-material alpha modes. The Viewer.set_alpha_mode function provides
+        a high-level interface to these settings. If you need more fine-grained control,
+        see the material.alpha_mode property of individual objects.
+
+        Parameters
+        ----------
+        mode :      str
+                    The mode to set. Please see the pygfx documentation for details:
+                      >>> import pygfx
+                      >>> help(pygfx.Material.alpha_mode)
+        objects :   list, optional
+                    Objects to set the alpha mode for. If None, will set for all objects.
+
+        """
+        if objects is None:
+            objects = list(self.objects)
+
+        for n in objects:
+            for v in self.objects[n]:
+                if getattr(v, "_pinned", False):
+                    continue
+                if not hasattr(v, "material"):
+                    continue
+                v.material.alpha_mode = mode
+
     @update_viewer(legend=True, bounds=False)
-    def set_colors(self, c):
+    def set_colors(self, c, alpha_mode="auto"):
         """Set object color.
 
         Parameters
         ----------
-        c :      tuple | dict
-                 RGB color(s) to apply. Values must be 0-1. Accepted:
+        c :     tuple | dict
+                RGB color(s) to apply. Values must be 0-1. Accepted:
                    1. Tuple of single color. Applied to all visible objects.
                    2. Dictionary names/IDs to colors.
+        alpha_mode : str
+                If "auto" (default), will set the alpha mode to "add" if the
+                opacity is < 1, and "opaque" otherwise. Set `alpha_mode` to `None` to
+                skip this adjustment.
 
         """
         objects = self.objects  # grab once to speed things up
@@ -1595,6 +1801,22 @@ class Viewer:
                         new_c = gfx.Color(cmap[n]).rgb
                     v.material.color = gfx.Color(new_c)
 
+                    # Determine if we consider this transparent
+                    if len(new_c) == 4 and new_c[3] < 1:
+                        is_transparent = True
+                    elif v.material.opacity < 1:
+                        is_transparent = True
+                    else:
+                        is_transparent = False
+
+                    if alpha_mode == "auto":
+                        if is_transparent:
+                            v.material.alpha_mode = "add"
+                        else:
+                            v.material.alpha_mode = "solid"
+                    elif alpha_mode:
+                        v.material.alpha_mode = alpha_mode
+
     def colorize(self, palette="seaborn:tab10", objects=None, randomize=True):
         """Colorize objects using a color palette.
 
@@ -1635,11 +1857,11 @@ class Viewer:
                 RGB(A) color to use for the background.
 
         """
-        self._background.set_colors(gfx.Color(c).rgba)
+        self._background.material.set_colors(gfx.Color(c).rgba)
 
     def _toggle_fps(self):
         """Switch FPS measurement on and off."""
-        self._show_fps = not self._show_fps
+        self.show_fps = not self.show_fps
 
     def screenshot(
         self, filename="screenshot.png", size=None, pixel_ratio=None, alpha=True
@@ -1648,7 +1870,7 @@ class Viewer:
 
         Parameters
         ----------
-        filename :      str, optional
+        filename :      str | pathlib.Path, optional
                         Filename to save to. If ``None``, will return image array.
                         Note that this will always save a PNG file, no matter
                         the extension.
@@ -1665,26 +1887,20 @@ class Viewer:
         """
         im = self._screenshot(alpha=alpha, size=size, pixel_ratio=pixel_ratio)
         if filename:
-            if not filename.endswith(".png"):
-                filename += ".png"
+            filename = Path(filename)
+            if filename.suffix != ".png":
+                filename = filename.parent / f"{filename.name}.png"
             png.from_array(
                 im.reshape(im.shape[0], im.shape[1] * im.shape[2]), mode="RGBA"
-            ).save(filename)
+            ).save(str(filename.resolve()))
         else:
             return im
 
     def _screenshot(self, alpha=True, size=None, pixel_ratio=None):
         """Return image array for screenshot."""
         if alpha:
-            oc = [
-                self._background.color_bottom_left,
-                self._background.color_bottom_right,
-                self._background.color_top_left,
-                self._background.color_top_right,
-            ]
-            self._background.set_colors((0, 0, 0, 0))
-            op = self._background.opacity
-            self._background.opacity = 0
+            vis = self._background.visible
+            self._background.visible = False
         if size:
             os = self.size
             self.size = size
@@ -1694,7 +1910,7 @@ class Viewer:
 
         # If this is an offscreen canvas, we need to manually trigger a draw first
         # Note: this has to happen _after_ adjust parameters!
-        if isinstance(self.canvas, WgpuCanvasOffscreen):
+        if isinstance(self.canvas, OffscreenRenderCanvas):
             self.canvas.draw()
         else:
             # This is a bit of a hack to make sure a new frame with the (potentially)
@@ -1709,8 +1925,7 @@ class Viewer:
             raise
         finally:
             if alpha:
-                self._background.set_colors(*oc)
-                self._background.opacity = op
+                self._background.visible = vis
             if size:
                 self.size = os
             if pixel_ratio:
@@ -1724,19 +1939,31 @@ class Viewer:
         Parameters
         ----------
         view :      XY | XZ | YZ | dict
-                    View to set. If a dictionary, should describe the
-                    state of the camera. Typically, this is obtained
-                    by calling `viewer.get_view()`.
+                    View to set. Can be inverted to e.g. "-XY" to show view from back.
+                    If a dictionary, should describe the state of the camera. Typically,
+                    this is obtained by calling `viewer.get_view()`.
 
         """
         if view == "XY":
             self.camera.show_object(
                 self.scene, view_dir=(0.0, 0.0, 1.0), up=(0.0, -1.0, 0.0)
             )
+        elif view == "-XY":
+            self.camera.show_object(
+                self.scene, view_dir=(0.0, 0.0, -1.0), up=(0.0, -1.0, 0.0)
+            )
         elif view == "XZ":
             self.camera.show_object(
                 self.scene, scale=1, view_dir=(0.0, 1.0, 0.0), up=(0.0, 0.0, 1.0)
             )
+        elif view == "-XZ":
+            self.camera.show_object(
+                self.scene, scale=1, view_dir=(0.0, -1.0, 0.0), up=(0.0, 0.0, 1.0)
+            )
+        elif view == "YZ":
+            self.camera.show_object(
+                self.scene, scale=1, view_dir=(1.0, 0.0, 0.0), up=(0.0, -1.0, 0.0)
+            )
         elif view == "YZ":
             self.camera.show_object(
                 self.scene, scale=1, view_dir=(-1.0, 0.0, 0.0), up=(0.0, -1.0, 0.0)
@@ -1777,10 +2004,10 @@ class Viewer:
             self._key_events[key] = func
         else:
             # We need to make `modifiers` is hashable
-            if isinstance(key, str):
-                key = (key,)
-            elif isinstance(key, (set, list)):
-                key = tuple(key)
+            if isinstance(modifiers, str):
+                modifiers = (modifiers,)
+            elif isinstance(modifiers, (set, list)):
+                modifiers = tuple(modifiers)
 
             if not isinstance(modifiers, tuple):
                 raise TypeError(
@@ -1821,3 +2048,26 @@ def handle_object_event(event, viewer, actions):
                 viewer.selected = np.append(viewer.selected, new_hover_id)
 
         logger.debug(f"Object: {new_hover_id}, Action: {actions}")
+
+
+def start_ipython_event_loop(gui):
+    ip = get_ipython()  # noqa
+    if not ip.active_eventloop:
+        try:
+            ip.enable_gui(gui)
+            logger.debug(
+                "Looks like you're running in an IPython environment but haven't "
+                "started a GUI event loop. We've started one for you using the "
+                f"{gui} backend."
+            )
+        except (ModuleNotFoundError, ImportError):
+            logger.warning(
+                "Looks like you're running an IPython environment but haven't "
+                "started a GUI event loop. We tried to start one for you using the "
+                f"{gui} backend (via %gui {gui}) but that failed. If you want a"
+                "non-blocking Octarine viewer, you may have to start the event loop "
+                "manually (see https://ipython.readthedocs.io/en/stable/config/eventloops.html)."
+                "Otherwise just call `Viewer.show()` to start a blocking viewer."
+            )
+            return False
+    return True
