@@ -5,6 +5,8 @@ import pygfx as gfx
 import numpy as np
 import trimesh as tm
 
+from importlib.util import find_spec
+
 from . import config, utils
 
 logger = config.get_logger(__name__)
@@ -34,7 +36,7 @@ def mesh2gfx(mesh, color, alpha=None):
     mat_color_kwargs, obj_color_kwargs = parse_mesh_color(mesh, color, alpha)
     # In theory we should be able to change pick_write on-the-fly since pygfx 0.3.0
     # But that doesn't seem to be the case.
-    mat_color_kwargs['pick_write'] = True
+    mat_color_kwargs["pick_write"] = True
 
     vis = gfx.Mesh(
         gfx.Geometry(
@@ -76,7 +78,7 @@ def geometry2gfx(geometry, color, alpha=None):
 
     # In theory we should be able to change pick_write on-the-fly since pygfx 0.3.0
     # But that doesn't seem to be the case.
-    mat_color_kwargs['pick_write'] = True
+    mat_color_kwargs["pick_write"] = True
 
     vis = gfx.Mesh(geometry, gfx.MeshPhongMaterial(**mat_color_kwargs))
 
@@ -91,6 +93,7 @@ def parse_mesh_color(mesh, color, alpha=None):
     """Parse color for mesh plotting."""
     mat_color_kwargs = dict()
     obj_color_kwargs = dict()
+    is_transparent = False
     if isinstance(color, np.ndarray) and color.ndim == 2:
         if alpha is not None:
             if color.shape[1] == 3:
@@ -99,6 +102,12 @@ def parse_mesh_color(mesh, color, alpha=None):
                 raise ValueError("Expected colors to have 3 or 4 channels.")
             color[:, -1] = alpha
 
+            if alpha < 1:
+                is_transparent = True
+        elif color.shape[1] == 4:
+            if np.any(color[:, -1] < 1):
+                is_transparent = True
+
         # Make sure the color is what pygfx expects
         if color.dtype in (np.float64,):
             color = color.astype(np.float32, copy=False)
@@ -118,8 +127,17 @@ def parse_mesh_color(mesh, color, alpha=None):
             color = gfx.Color(color).rgba
             color = (color[0], color[1], color[2], alpha)
 
+            if alpha < 1:
+                is_transparent = True
+        elif isinstance(color, (list, np.ndarray, tuple)) and (len(color) == 4):
+            is_transparent = color[3] < 1
+
         mat_color_kwargs["color"] = color
 
+    if is_transparent:
+        # The default blend mode is "auto" which will show transparency but is a bit ugly
+        mat_color_kwargs["alpha_mode"] = "add"
+
     return mat_color_kwargs, obj_color_kwargs
 
 
@@ -267,7 +285,7 @@ def volume2gfx(
                 clim=(cmin, cmax),
                 map=cmap,
                 interpolation=interpolation,
-                map_interpolation=interpolation,
+                alpha_mode="add",
             ),
         )
         visuals.append(vis)
@@ -316,43 +334,68 @@ def volume2gfx(
 
 
 def to_colormap(x, hide_zero):
-    """Convert `x` to a gfx.Texture that can be used for Volumes."""
-    # If this is a texture
+    """Convert `x` to a gfx.TextureMap that can be used for Volumes."""
+    # If x is None, use the default colormap
     if x is None:
-        tex = gfx.cm.cividis
+        tm = gfx.cm.cividis
+    # If this is a texture
     elif isinstance(x, gfx.Texture):
         if x.dim != 1:
             raise ValueError("Expected 1D texture.")
-        tex = x
+        tm = gfx.TextureMap(x)
+    # If this is a texturemap
+    elif isinstance(x, gfx.TextureMap):
+        if x.texure.dim != 1:
+            raise ValueError("Expected 1D texture.")
+        tm = x
     elif isinstance(x, str) and hasattr(gfx.cm, x):
-        tex = getattr(gfx.cm, x)
+        tm = getattr(gfx.cm, x)
     elif isinstance(x, gfx.Color):
         # cmap needs a list of colors (even if len == 1)
-        tex = cmap.Colormap([x.rgba]).to_pygfx()
+        tm = cmap.Colormap([x.rgba]).to_pygfx()
     elif isinstance(x, cmap.Colormap):
-        tex = x.to_pygfx()
+        tm = x.to_pygfx()
     elif isinstance(x, (dict, list)):
         # cmap can interpret dict and list of colors
-        tex = cmap.Colormap(x).to_pygfx()
+        tm = cmap.Colormap(x).to_pygfx()
+    elif isinstance(x, str):
+        tm = cmap.Colormap(x).to_pygfx()
     else:
         # Last ditch effort: see if cmap can handle it
-        tex = cmap.Colormap([x]).to_pygfx()
+        c = cmap.Colormap([x])
+
+        # If x is a single (RGB) color, cmap will create a colormap
+        # with the first color being `None` and the second being `x`.
+        # We need to set the first color to black
+        if len(c.color_stops) == 2 and c.color_stops[0].color == "none":
+            c = cmap.Colormap(["k", x])
+
+        tm = c.to_pygfx()
 
     if hide_zero:
         # Add an alpha column if needed
-        if tex.data.shape[1] == 3:
-            colors = np.hstack(
-                (tex.data, np.ones((tex.data.shape[0], 1))), dtype=tex.data.dtype
-            )
-            tex = gfx.Texture(colors, dim=1)
+        if tm.texture.data.shape[1] == 3:
+            np_ver = [int(i) for i in np.__version__.split(".")]
+            # Prior to version 1.24.0, numpy's hstack did not accept a `dtype`
+            # parameter directly
+            if np_ver[0] <= 1 and np_ver[1] < 24:
+                colors = np.hstack(
+                    (tm.texture.data, np.ones((tm.texture.data.shape[0], 1)))
+                ).astype(tm.texture.data.dtype)
+            else:
+                colors = np.hstack(
+                    (tm.texture.data, np.ones((tm.texture.data.shape[0], 1))),
+                    dtype=tm.texture.data.dtype,
+                )
+            tm = gfx.TextureMap(gfx.Texture(colors, dim=1))
         # Otherwise make a copy to avoid modifying the original data
         else:
-            tex = gfx.Texture(tex.data.copy(), dim=1)
+            tm = gfx.TextureMap(gfx.Texture(tm.texture.data.copy(), dim=1))
 
         # Set alpha channel for first color to 0
-        tex.data[0, 3] = 0
+        tm.texture.data[0, 3] = 0
 
-    return tex
+    return tm
 
 
 def points2gfx(points, color, size=2, marker=None, size_space="screen"):
@@ -398,7 +441,7 @@ def points2gfx(points, color, size=2, marker=None, size_space="screen"):
     material_kwargs = {}
     # In theory we should be able to change pick_write on-the-fly since pygfx 0.3.0
     # But that doesn't seem to be the case.
-    material_kwargs['pick_write'] = True
+    material_kwargs["pick_write"] = True
 
     # Parse sizes
     if utils.is_iterable(size):
@@ -500,13 +543,13 @@ def lines2gfx(lines, color, linewidth=1, linewidth_space="screen", dash_pattern=
     if dash_pattern is None:
         dash_pattern = ()  # pygfx expects an empty tuple for solid lines
     elif isinstance(dash_pattern, str):
-        if dash_pattern == "solid":
+        if dash_pattern in ("solid", "-"):
             dash_pattern = ()
-        elif dash_pattern == "dashed":
+        elif dash_pattern in ("dashed", "--"):
             dash_pattern = (5, 2)
-        elif dash_pattern == "dotted":
+        elif dash_pattern in ("dotted", ":"):
             dash_pattern = (1, 2)
-        elif dash_pattern == "dashdot":
+        elif dash_pattern in ("dashdot", "-."):
             dash_pattern = (5, 2, 1, 2)
         else:
             raise ValueError(f"Unknown dash pattern: {dash_pattern}")
@@ -516,7 +559,7 @@ def lines2gfx(lines, color, linewidth=1, linewidth_space="screen", dash_pattern=
 
     # In theory we should be able to change pick_write on-the-fly since pygfx 0.3.0
     # But that doesn't seem to be the case.
-    material_kwargs['pick_write'] = True
+    material_kwargs["pick_write"] = True
 
     # Parse color(s)
     if isinstance(color, np.ndarray) and color.ndim == 2:
@@ -579,8 +622,11 @@ def trimesh2gfx(mesh, color=None, alpha=None, use_material=True):
     kwargs = dict(
         positions=np.ascontiguousarray(mesh.vertices, dtype="f4"),
         indices=np.ascontiguousarray(mesh.faces, dtype="i4"),
-        normals=np.ascontiguousarray(mesh.vertex_normals, dtype="f4"),
     )
+    # trimesh needs scipy to compute normals
+    if find_spec("scipy"):
+        kwargs["normals"] = np.ascontiguousarray(mesh.vertex_normals, dtype="f4")
+
     if mesh.visual.kind == "texture" and getattr(mesh.visual, "uv", None) is not None:
         # convert the uv coordinates from opengl to wgpu conventions.
         # wgpu uses the D3D and Metal coordinate systems.
@@ -630,7 +676,7 @@ def simple_material_from_trimesh(material):
     gfx_material = gfx.MeshPhongMaterial(
         color=material.ambient / 255,
         pick_write=True,  # we can't seem to change this on-the-fly
-        )
+    )
 
     gfx_material.shininess = material.glossiness
     gfx_material.specular = gfx.Color(*(material.specular / 255))
@@ -683,9 +729,9 @@ gfx.materials._compat.texture_from_pillow_image = texture_from_pillow_image
 
 def scene2gfx(scene):
     """Convert trimesh Scene to pygfx visuals."""
-    assert isinstance(
-        scene, tm.scene.scene.Scene
-    ), f"Expected trimesh scene, got {type(scene)}."
+    assert isinstance(scene, tm.scene.scene.Scene), (
+        f"Expected trimesh scene, got {type(scene)}."
+    )
 
     # Get all the geometry names
     gfx_geometries = {}
@@ -712,7 +758,7 @@ def text2gfx(
     position=(0, 0, 0),
     color="w",
     font_size=1,
-    anchor="topright",
+    anchor="top-right",
     screen_space=False,
     markdown=False,
 ):
@@ -729,9 +775,10 @@ def text2gfx(
     font_size :     int, optional
                     Font size.
     anchor :        str, optional
-                    Anchor point of the text. Can be one of "topleft", "topright",
-                    "bottomleft", "bottomright", "center", "topmiddle", "bottommiddle", "middleleft",
-                    "middleright".
+                    Anchor point of the text. Combination of vertical and
+                    horizontal alignment (e.g. "top-center"):
+                     - vertical: "top", "bottom", "middle", "baseline"
+                     - horizontal: "left", "right", "center"
     screen_space :  bool, optional
                     Whether to use screen space coordinates.
     markdown :      bool, optional
@@ -758,8 +805,8 @@ def text2gfx(
         defaults["text"] = text
 
     text = gfx.Text(
-        gfx.TextGeometry(**defaults),
-        gfx.TextMaterial(color=color),
+        **defaults,
+        material=gfx.TextMaterial(color=color),
     )
     text.local.position = position
     return text
@@ -772,3 +819,75 @@ def visual_passthrough(x, *args, **kwargs):
             "Pygfx visuals are passed-through as is. Any additional arguments are ignored."
         )
     return x
+
+
+class CloudImage:
+    def __init__(self, volume, viewer, slice=None):
+        self.volume = volume
+        self.volume.bounded = False
+        self.volume.parallel = 10
+        self.viewer = viewer
+        self._slice = (
+            slice if slice is not None else volume.scales[volume.mip]["voxel_offset"][2]
+        )
+        self._mip = self.volume.mip
+
+        self.tex = gfx.Texture(np.zeros(self.shape, dtype=self.volume.dtype), dim=2)
+        self.image = gfx.Image(
+            gfx.Geometry(grid=self.tex),
+            gfx.ImageBasicMaterial(clim=(0, 255)),
+        )
+
+        self.update_image()  # this updates the content of the image
+        self.viewer.add(self.image)
+
+    @property
+    def mip(self):
+        return self._mip
+
+    @mip.setter
+    def mip(self, value):
+        if self._mip != value:
+            # Translate the old slice to the new mip level
+            self._slice = int(
+                self._slice
+                * self.volume.scales[self._mip]["resolution"][2]
+                / self.volume.scales[value]["resolution"][2]
+            )
+
+            # Update the mip levels
+            self._mip = value
+            self.volume.mip = value
+
+            # Update the texture
+            self.update_texture()
+            self.update_image()
+
+    @property
+    def shape(self):
+        return self.volume.shape[:2]
+
+    @property
+    def slice(self):
+        return self._slice
+
+    @slice.setter
+    def slice(self, value):
+        if self._slice != value:
+            self._slice = value
+            self.update_image()
+
+    @property
+    def resolution(self):
+        return self.volume.scales[self.volume.mip]["resolution"]
+
+    def update_image(self):
+        self.image.local.scale = self.resolution
+        self.image.local.z = self.resolution[2] * self.slice
+
+        self.tex.data[:] = self.volume[:, :, self.slice].reshape(self.shape)
+        self.tex.update_range((0, 0, 0), self.tex.size)
+
+    def update_texture(self):
+        self.tex = gfx.Texture(np.zeros(self.shape, dtype=self.volume.dtype), dim=2)
+        self.image.geometry.grid = self.tex
